import {ErrorHandlingComponent, SubscribingComponent} from '../shared/components/subscribing.component';
import {Observable} from 'rxjs';
import {ComboBoxEntry} from '../shared/components/inputs/inputs';
import {map, tap} from 'rxjs/operators';
import {RecipeService} from './services/recipe.service';
import {CompanyService} from '../company/service/company.service';
import {AppState} from '../shared/app-state';
import {ErrorService} from '../shared/service/error.service';
import {ActivatedRoute, Router} from '@angular/router';
import {FormControl, Validators} from '@angular/forms';
import {Component, EventEmitter, Input, Output} from '@angular/core';
import {Recipe} from '../shared/model/recipe.model';
import {AccountService} from '../accounts/services/account.service';
import {Permission} from '../shared/model/user';

@Component({
  selector: 'cre-recipe-add',
  templateUrl: 'add.html'
})
export class RecipeAdd extends ErrorHandlingComponent {
  controls: any
  companyEntries$: Observable<ComboBoxEntry[]> = this.companyService.all.pipe(
    map(companies => companies.map(c => new ComboBoxEntry(c.id, c.name))),
  )

  errorHandlers = [{
    filter: error => error.type === `exists-recipe-company-name`,
    messageProducer: error => `Une couleur avec le nom ${error.name} existe déjà pour la bannière ${error.company}`
  }]

  constructor(
    private recipeService: RecipeService,
    private companyService: CompanyService,
    private appState: AppState,
    errorService: ErrorService,
    router: Router,
    activatedRoute: ActivatedRoute
  ) {
    super(errorService, activatedRoute, router)
    this.appState.title = 'Nouvelle couleur'
  }

  submit(recipe: Recipe) {
    this.subscribe(
      this.recipeService.save(recipe),
      recipe => this.urlUtils.navigateTo(`/color/edit/${recipe.id}`)
    )
  }
}

@Component({
  selector: 'recipe-form',
  templateUrl: 'form.html'
})
export class RecipeForm extends SubscribingComponent {
  @Input() recipe: Recipe | null

  @Output() submitForm = new EventEmitter<Recipe>();

  controls: any
  companyEntries$: Observable<ComboBoxEntry[]>
  hasCompanies = true

  constructor(
    private companyService: CompanyService,
    private accountService: AccountService,
    errorService: ErrorService,
    activatedRoute: ActivatedRoute,
    router: Router,
  ) {
    super(errorService, activatedRoute, router)
  }

  ngOnInit() {
    super.ngOnInit();

    this.fetchCompanies()

    this.controls = {
      name: new FormControl(null, Validators.required),
      description: new FormControl(null, Validators.required),
      color: new FormControl('#ffffff', Validators.required),
      gloss: new FormControl(0, Validators.compose([Validators.required, Validators.min(0), Validators.max(100)])),
      sample: new FormControl(null, Validators.compose([Validators.required, Validators.min(0)])),
      approbationDate: new FormControl(null),
      remark: new FormControl(null),
      company: new FormControl(null, Validators.required)
    }
  }

  private fetchCompanies() {
    this.companyEntries$ = this.companyService.all.pipe(
      tap(companies => this.hasCompanies = companies.length > 0),
      map(companies => companies.map(c => new ComboBoxEntry(c.id, c.name))),
    )
  }

  submit() {
    this.submitForm.emit({
      id: this.recipe?.id,
      name: this.controls.name.value,
      description: this.controls.description.value,
      color: this.controls.color.value,
      gloss: this.controls.gloss.value,
      sample: this.controls.sample.value,
      approbationDate: this.controls.approbationDate.value,
      remark: this.controls.remark.value,
      company: this.controls.company.value,
      mixes: this.recipe?.mixes,
      approbationExpired: this.recipe?.approbationExpired,
      groupsInformation: this.recipe?.groupsInformation,
      imagesUrls: this.recipe?.imagesUrls
    })
  }

  get hasCompanyEditPermission(): boolean {
    return this.accountService.hasPermission(Permission.EDIT_COMPANIES)
  }
}
